package lib;

import lib.ui.elements.UIElement;
import lib.ui.elements.TextElement;
import lib.ui.elements.RootElement;
import kernel.KernelEvents;
import bin.Terminal;
import kernel.ui.WindowManager;
import kernel.ui.WindowContext;

/**
	The WindowContext that spawns all other contexts. This is the main terminal.
	Listens to global hotkey "POS1"/"HOME" to switch back to this context.
**/
class HomeContext {
	private static inline final MAX_CONTEXT:Int = 10;

	private var ctx:WindowContext = null;
	private final workspaces:Map<Int, WindowContext> = [];
	private var currentWorkspace:Int = -1;
	private var requestRender: Void->Void;
	private var renderer:RootElement;

	public function new() {}

	public function run() {
		// Create main terminal context
		var stateless = WindowManager.instance.createNewStatelessContext();
		ctx = stateless.ctx;
		requestRender = stateless.requestRender;
		WindowManager.instance.focusContextToOutput(ctx, "main");

		renderer = new RootElement();

		ctx.delegateEvents(renderer);
		stateless.setRenderFunc(this.render);
		requestRender();

		// Register global key bindings to react to main terminal
		KernelEvents.instance.onKey.handle(e -> {
			// Is HOME pressed?
			// TODO: remove magic number
			if (e.keyCode == 268) {
				focusMainTerm();
			}
		});
	}

	/**
		Adds a context to the list of contexts to manage. Returns the context ID. Returns -1 if the context could not be added.
	**/
	private function addContextNextWorkspace(context:WindowContext):Int {
		for (i in 0...MAX_CONTEXT) {
			if (!workspaces.exists(i)) {
				workspaces[i] = context;
				return i;
			}
		}

		return -1;
	}

	private function addContextToWorkspace(context:WindowContext, id:Int) {
		if (!workspaces.exists(id)) {
			workspaces[id] = context;
		}
	}

	private function focusContext(id:Int) {
		if (workspaces.exists(id)) {
			WindowManager.instance.focusContextToOutput(workspaces[id], "main");
			currentWorkspace = id;
		}
	}

	private function focusMainTerm() {
		requestRender();
		WindowManager.instance.focusContextToOutput(ctx, "main");
		currentWorkspace = -1;
	}

	private function handleSelectContext(contextId:Int) {
		if (workspaces[contextId] != null) {
			focusContext(contextId);
		} else {
			var newContext = WindowManager.instance.createNewBufferedContext();

			// Create new terminal
			var term = new Terminal();
			term.invoke(newContext);

			addContextToWorkspace(newContext, contextId);
			focusContext(contextId);
		}
	}

	private function render() {
		ctx.clear();
		ctx.setCursorBlink(false);

		var list = [
			for (i in 0...MAX_CONTEXT) workspaces.exists(i) ? 'Switch to context ${i}' : 'Create new Terminal on context ${i}'
		];

		var children:Array<UIElement> = [
			for (i => line in list) new TextElement(line, {onClick: this.handleSelectContext.bind(i)})
		];

		renderer.setChildren(children);

		renderer.render().renderToContext(ctx);
	}
}
